// SoundSampler.java
// Compiles J2SE V5 up

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.sound.sampled.*;

public class SoundSampler extends JFrame
        implements ActionListener
{
  enum State
  {
    IDLE, RECORDING, STOPPING, PLAYING, SAVING;
  }

  private SoundRecorder recorder =
    new SoundRecorder(MyAudioFormat.getAudioFormat());
  private StreamingBufferPlayer player =
    new StreamingBufferPlayer(MyAudioFormat.getAudioFormat());
  private AudioFormat audioFormat;
  private final JButton[] buttons = {new JButton("Capture"),
                                     new JButton("Stop"),
                                     new JButton("Playback"),
                                     new JButton("Save")};
  private final int nbButtons = buttons.length;
  private State state = State.IDLE;
  private ByteArrayOutputStream data =
    new ByteArrayOutputStream();

  public SoundSampler()
  {
    for (int i = 0; i < nbButtons; i++)
    {
      buttons[i].addActionListener(this);
      getContentPane().add(buttons[i]);
    }

    getContentPane().setLayout(new FlowLayout());
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    showTitle("Sound Sampler");
    setSize(350, 70);
    setLocation(200, 100);
    enableButtons(new boolean[]{true, false, false, false});
    setVisible(true);
    new Thread()
    {
      public void run()
      {
        eventLoop();
      }
    }.start();
  }

  private void eventLoop()
  {
    while (true) // Event loop
    {
      try
      {
        switch (state)
        {
          case IDLE :
            break;

          case RECORDING :
            showTitle("Recording...");
            enableButtons(new boolean[] {false, true, false, false});
            data.reset();
            recorder.capture(data); // Returns immediately
            state = State.IDLE;
            break;

          case STOPPING :
            showTitle("Stopped");
            recorder.setStopCapture(true);
            enableButtons(new boolean[] {true, false, true, true});
            state = State.IDLE;
            break;

          case PLAYING :
            showTitle("Playing...");
            enableButtons(new boolean[] {false, false, false, false});
            player.start(data, true); // Play until finished
            showTitle("Stopped");
            enableButtons(new boolean[] {true, false, true, true});
            state = State.IDLE;
            break;

          case SAVING :
            enableButtons(new boolean[] {false, false, false, false});
            JFileChooser chooser = new JFileChooser();
            int rc = chooser.showSaveDialog(null);
            if (rc == JFileChooser.APPROVE_OPTION)
            {
              showTitle("Saving...");
              recorder.writeWavFile(
                data, chooser.getSelectedFile());
            }
            showTitle("Stopped");
            enableButtons(new boolean[] {true, false, true, true});
            state = State.IDLE;
            break;
        }
      }
      catch (LineUnavailableException ex)
      {
        System.out.println("Sound card not available");
        System.exit(1);
      }
    }
  }

  private void enableButtons(boolean[] b)
  {
    for (int i = 0; i < b.length; i++)
      buttons[i].setEnabled(b[i]);
  }

  private void showTitle(final String text)
  // Invoke Swing methods in EDT
  {
    EventQueue.invokeLater(new Runnable()
    {
      public void run()
      {
        setTitle(text);
      }
    });
  }

  public void actionPerformed(ActionEvent evt)
  {
    Object source = evt.getSource();
    if (source == buttons[0])
    {
      state = State.RECORDING;
    }
    if (source == buttons[1])
    {
      state = State.STOPPING;
    }
    if (source == buttons[2])
    {
      state = State.PLAYING;
    }
    if (source == buttons[3])
    {
      state = State.SAVING;
    }
  }

  public static void main(String args[])
  {
    EventQueue.invokeLater(new Runnable()
    {
      public void run()
      {
        new SoundSampler();
      }
    });
  }
}
